import cloudwatch = require('@aws-cdk/aws-cloudwatch');
import { Construct } from '@aws-cdk/core';
import { IFunction, QualifiedFunctionBase } from './function-base';
export interface IVersion extends IFunction {
    /**
     * The most recently deployed version of this function.
     * @attribute
     */
    readonly version: string;
    /**
     * The underlying AWS Lambda function.
     */
    readonly lambda: IFunction;
}
/**
 * Properties for a new Lambda version
 */
export interface VersionProps {
    /**
     * SHA256 of the version of the Lambda source code
     *
     * Specify to validate that you're deploying the right version.
     *
     * @default No validation is performed
     */
    readonly codeSha256?: string;
    /**
     * Description of the version
     *
     * @default Description of the Lambda
     */
    readonly description?: string;
    /**
     * Function to get the value of
     */
    readonly lambda: IFunction;
}
export interface VersionAttributes {
    /**
     * The version.
     */
    readonly version: string;
    /**
     * The lambda function.
     */
    readonly lambda: IFunction;
}
/**
 * A single newly-deployed version of a Lambda function.
 *
 * This object exists to--at deploy time--query the "then-current" version of
 * the Lambda function that it refers to. This Version object can then be
 * used in `Alias` to refer to a particular deployment of a Lambda.
 *
 * This means that for every new update you deploy to your Lambda (using the
 * CDK and Aliases), you must always create a new Version object. In
 * particular, it must have a different name, so that a new resource is
 * created.
 *
 * If you want to ensure that you're associating the right version with
 * the right deployment, specify the `codeSha256` property while
 * creating the `Version.
 */
export declare class Version extends QualifiedFunctionBase implements IVersion {
    /**
     * Construct a Version object from a Version ARN.
     *
     * @param scope The cdk scope creating this resource
     * @param id The cdk id of this resource
     * @param versionArn The version ARN to create this version from
     */
    static fromVersionArn(scope: Construct, id: string, versionArn: string): IVersion;
    static fromVersionAttributes(scope: Construct, id: string, attrs: VersionAttributes): IVersion;
    readonly version: string;
    readonly lambda: IFunction;
    readonly functionArn: string;
    readonly functionName: string;
    protected readonly canCreatePermissions = true;
    constructor(scope: Construct, id: string, props: VersionProps);
    readonly grantPrincipal: import("@aws-cdk/aws-iam").IPrincipal;
    readonly role: import("@aws-cdk/aws-iam").IRole | undefined;
    metric(metricName: string, props?: cloudwatch.MetricOptions): cloudwatch.Metric;
}
